"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.api = exports.Api = void 0;
const equality_1 = require("../equality");
class Api {
    constructor(baseURL, init, before, after) {
        this.baseURL = baseURL;
        this.init = init;
        this.before = before;
        this.after = after;
    }
    /**
     * fetchを実行する
     * @param path baseURLに続くパス
     * @param init fetchのinit
     */
    fetch(path, init) {
        return __awaiter(this, void 0, void 0, function* () {
            this.before();
            const response = yield fetch(`${this.baseURL}${path}`, this.mergeInit(init));
            this.after();
            return response;
        });
    }
    get(path, init) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.methodFetch("GET", path, init);
        });
    }
    post(path, init) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.methodFetch("POST", path, init);
        });
    }
    patch(path, init) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.methodFetch("PATCH", path, init);
        });
    }
    put(path, init) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.methodFetch("PUT", path, init);
        });
    }
    delete(path, init) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.methodFetch("DELETE", path, init);
        });
    }
    create(path, init, before, after) {
        return new Api(`${this.baseURL}${path}`, this.mergeInit(init), this.mergeBefore(before), this.mergeAfter(after));
    }
    equals(other) {
        return (this.baseURL === other.baseURL &&
            (0, equality_1.equals)(this.init, other.init) &&
            this.before === other.before &&
            this.after === other.after);
    }
    get clone() {
        return new Api(this.baseURL, Object.create(this.init), this.before, this.after);
    }
    methodFetch(method, path, init) {
        return this.fetch(`${this.baseURL}${path}`, Object.assign(Object.assign({}, init), { method }));
    }
    mergeInit(init) {
        if (init == null)
            return this.init;
        return Object.assign(Object.assign({}, this.init), { headers: this.mergeHeaders(init.headers) });
    }
    mergeHeaders(headersInit) {
        if (headersInit == null)
            return new Headers(this.init.headers);
        const headers = new Headers(this.init.headers);
        for (const [key, value] of new Headers(headersInit)) {
            headers.set(key, value);
        }
        return headers;
    }
    mergeBefore(before) {
        if (before == null)
            return this.before;
        return () => {
            this.before();
            before();
        };
    }
    mergeAfter(after) {
        if (after == null)
            return this.after;
        return () => {
            after();
            this.after();
        };
    }
}
exports.Api = Api;
function api(args) {
    var _a, _b, _c, _d;
    return new Api((_a = args.baseURL) !== null && _a !== void 0 ? _a : "", (_b = args.init) !== null && _b !== void 0 ? _b : {}, (_c = args.before) !== null && _c !== void 0 ? _c : (() => { }), (_d = args.after) !== null && _d !== void 0 ? _d : (() => { }));
}
exports.api = api;
//# sourceMappingURL=api.js.map