"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decode = exports.isDecodable = void 0;
const clone_1 = require("../clone");
function isDecodable(object) {
    return (typeof object === "object" &&
        object !== null &&
        "decode" in object &&
        typeof object.decode === "function");
}
exports.isDecodable = isDecodable;
function decode(json, origin) {
    if (json == null && origin == null) {
        return origin;
    }
    if (typeof json !== typeof origin) {
        throw new Error(`Decode Type Error! expected ${typeof json} but got ${origin} of ${typeof origin}`);
    }
    if (typeof json !== "object") {
        return json;
    }
    if (isDecodable(origin)) {
        return origin.decode(json);
    }
    if (Array.isArray(origin)) {
        if (!Array.isArray(json)) {
            throw new Error(`Decode Type Error! expected Array but got ${json}`);
        }
        if (origin.length === 0) {
            throw new Error(`Decode Type Error! should be non-empty Array`);
        }
        return json.map((childJson) => decode(childJson, origin[0]));
    }
    const newObject = (0, clone_1.clone)(origin);
    for (const key in json) {
        if (!(key in newObject))
            continue;
        newObject[key] = decode(json[key], newObject[key]);
    }
    return newObject;
}
exports.decode = decode;
//# sourceMappingURL=iDecodable.js.map