"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compareTo = exports.compare = exports.isComparable = exports.isComparableObject = void 0;
const equality_1 = require("../equality");
function isComparableObject(value) {
    return (value != null &&
        typeof value === "object" &&
        "compare" in value &&
        typeof value.compare === "function");
}
exports.isComparableObject = isComparableObject;
function isComparable(value) {
    if (typeof value === "string")
        return true;
    if (typeof value === "number")
        return true;
    if (typeof value === "bigint")
        return true;
    return isComparableObject(value);
}
exports.isComparable = isComparable;
function compare(a, b) {
    if (!isComparable(a)) {
        throw new Error(`should be comparable but: ${a}`);
    }
    if (isComparableObject(a)) {
        return a.compare(b);
    }
    return a < b;
}
exports.compare = compare;
function compareTo(a, b) {
    if ((0, equality_1.equals)(a, b))
        return 0;
    return compare(a, b) ? -1 : 1;
}
exports.compareTo = compareTo;
//# sourceMappingURL=iComparable.js.map