"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const iComparable_1 = require("./iComparable");
(0, vitest_1.describe)("isComparable Tests", () => {
    (0, vitest_1.test)("string, number, bigintは比較可能", () => {
        (0, vitest_1.expect)((0, iComparable_1.isComparable)("")).toBeTruthy();
        (0, vitest_1.expect)((0, iComparable_1.isComparable)(1)).toBeTruthy();
        (0, vitest_1.expect)((0, iComparable_1.isComparable)(BigInt(3))).toBeTruthy();
    });
    (0, vitest_1.test)("通常のオブジェクトは比較できない", () => {
        (0, vitest_1.expect)((0, iComparable_1.isComparable)({})).toBeFalsy();
    });
    (0, vitest_1.test)("compareメソッドを持つオブジェクトは比較可能", () => {
        (0, vitest_1.expect)((0, iComparable_1.isComparable)({ compare: () => true })).toBeTruthy();
    });
    (0, vitest_1.test)("compareプロパティを持つオブジェクトは比較不可", () => {
        (0, vitest_1.expect)((0, iComparable_1.isComparable)({ compare: "" })).toBeFalsy();
    });
    (0, vitest_1.test)("compareToで同じならば0を返す", () => {
        const a = { compare: () => false, equals: () => true };
        const b = { compare: () => true, equals: () => true };
        (0, vitest_1.expect)((0, iComparable_1.compareTo)(a, b)).toBe(0);
    });
    (0, vitest_1.test)("左の方が小さいならばで同じならば-1を返す", () => {
        const a = { compare: () => true, equals: () => false };
        const b = { compare: () => false, equals: () => false };
        (0, vitest_1.expect)((0, iComparable_1.compareTo)(a, b)).toBe(-1);
    });
    (0, vitest_1.test)("右の方が小さいならばで同じならば1を返す", () => {
        const a = { compare: () => false, equals: () => false };
        const b = { compare: () => true, equals: () => false };
        (0, vitest_1.expect)((0, iComparable_1.compareTo)(a, b)).toBe(1);
    });
});
//# sourceMappingURL=iComparable.test.js.map