import { Hashable, IHashable, IHashableObject } from "../hash";
import { IEquatable, IEquatableObject } from "../equality";
import { Set } from "../set/set";
import { Sequence } from "../sequence";
import { None, Optional } from "../optional";
import { IDecodable, IEncodable } from "../codable";
import { IClonable } from "../clone";
export type DictionaryKey = IHashable & IEquatable;
export declare class Dictionary<K extends DictionaryKey, V> implements IEquatableObject<Dictionary<K, V>>, IHashableObject<Dictionary<K, V>>, IClonable<Dictionary<K, V>>, IEncodable, IDecodable<Dictionary<K, V>> {
    private readonly hashKV;
    constructor(hashKV: Map<string, Set<KV<K, V>>>);
    /** 引数のキーに対応する値を返す */
    get(key: K): Optional<V>;
    /** 対応するキーの値を更新する。none系を渡すとキーが削除される */
    put(key: K, value: V | null | undefined | None<V>): Dictionary<K, V>;
    private get shallowCopyHashKV();
    equals(other: Dictionary<K, V>): boolean;
    get hashValue(): string;
    map<NK extends IEquatable & IHashable, NU>(f: (key: K, value: V) => [NK, NU]): Dictionary<NK, NU>;
    filter(f: (key: K, value: V) => boolean): Dictionary<K, V>;
    every(f: (key: K, value: V) => boolean): boolean;
    some(f: (key: K, value: V) => boolean): boolean;
    get encode(): Record<string | number, unknown>;
    decode(object: unknown): Dictionary<K, V>;
    get clone(): Dictionary<K, V>;
    get isEmpty(): boolean;
    get nonEmpty(): boolean;
    get count(): number;
    get keys(): Sequence<K>;
    get values(): Sequence<V>;
    [Symbol.iterator](): IterableIterator<[K, V]>;
    private get allKvSet();
    private get allKvArrays();
    private get allKvSeq();
}
export declare class KV<K extends DictionaryKey, V> extends Hashable<KV<K, V>> {
    readonly key: K;
    readonly value: V;
    constructor(key: K, value: V);
    get properties(): readonly IEquatable[];
}
export declare function kv<K extends DictionaryKey, V>(key: K, value: V): KV<K, V>;
export declare function dict<K extends string | number, V>(record: Record<K, V>): Dictionary<K, V>;
export declare function dict<K extends DictionaryKey, V>(array: readonly KV<K, V>[] | Sequence<KV<K, V>>): Dictionary<K, V>;
export declare function dict<K extends DictionaryKey, V>(): Dictionary<K, V>;
