"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const dictionary_1 = require("./dictionary");
const optional_1 = require("../optional");
const set_1 = require("../set/set");
const sequence_1 = require("../sequence");
const assertion_test_1 = require("../equality/assertion.test");
(0, vitest_1.describe)("Dictionary Tests", () => {
    (0, vitest_1.test)("置いたものを取り出せる", () => {
        let dictionary = (0, dictionary_1.dict)();
        dictionary = dictionary.put("a", 1);
        (0, vitest_1.expect)(dictionary.get("a")).toEqual((0, optional_1.some)(1));
    });
    (0, vitest_1.test)("値を更新できる", () => {
        let dictionary = (0, dictionary_1.dict)({
            hello: "world",
        });
        dictionary = dictionary.put("hello", "こんにちは");
        (0, vitest_1.expect)(dictionary.get("hello")).toEqual((0, optional_1.some)("こんにちは"));
    });
    (0, vitest_1.test)("nullまたはnoneまたはundefinedで更新すると消せる", () => {
        let dictionary = (0, dictionary_1.dict)({
            hello: "world",
        });
        const nullDictionary = dictionary.put("hello", null);
        (0, vitest_1.expect)(nullDictionary.get("hello")).toBe((0, optional_1.none)());
        const undefinedDictionary = dictionary.put("hello", undefined);
        (0, vitest_1.expect)(undefinedDictionary.get("hello")).toBe((0, optional_1.none)());
        const noneDictionary = dictionary.put("hello", (0, optional_1.none)());
        (0, vitest_1.expect)(noneDictionary.get("hello")).toBe((0, optional_1.none)());
    });
    (0, vitest_1.test)("decodable", () => {
        const json = {
            hello: true,
            world: false,
        };
        const origin = (0, dictionary_1.dict)([(0, dictionary_1.kv)("", true)]);
        const actual = origin.decode(json);
        (0, vitest_1.expect)(actual.get("hello")).toEqual((0, optional_1.some)(true));
        (0, vitest_1.expect)(actual.get("world")).toEqual((0, optional_1.some)(false));
    });
    (0, vitest_1.test)("iterable", () => {
        const dictionary = (0, dictionary_1.dict)({
            hello: "こんにちは",
            world: "世界",
        });
        let keys = (0, set_1.set)([]);
        let values = (0, set_1.set)([]);
        for (const [key, value] of dictionary) {
            keys = keys.insert(key);
            values = values.insert(value);
        }
        (0, vitest_1.expect)(keys).toEqual((0, set_1.set)(["hello", "world"]));
        (0, vitest_1.expect)(values).toEqual((0, set_1.set)(["こんにちは", "世界"]));
    });
    (0, vitest_1.test)("values", () => {
        const dictionary = (0, dictionary_1.dict)({
            hello: "こんにちは",
            world: "世界",
        });
        (0, vitest_1.expect)(dictionary.values).toEqual((0, sequence_1.seq)(["こんにちは", "世界"]));
    });
    (0, vitest_1.test)("keys", () => {
        const dictionary = (0, dictionary_1.dict)({
            hello: "こんにちは",
            world: "世界",
        });
        (0, vitest_1.expect)(dictionary.keys).toEqual((0, sequence_1.seq)(["hello", "world"]));
    });
    (0, vitest_1.test)("map", () => {
        const dictionary = (0, dictionary_1.dict)({
            hello: "こんにちは",
            world: "世界",
        });
        const expected = (0, dictionary_1.dict)({
            "hello?": "こんにちは!",
            "world?": "世界!",
        });
        const actual = dictionary.map((key, value) => [key + "?", value + "!"]);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("filter", () => {
        const dictionary = (0, dictionary_1.dict)({
            hello: "こんにちは",
            world: "世界",
        });
        const expected = (0, dictionary_1.dict)({
            hello: "こんにちは",
        });
        const actual = dictionary.filter((key, value) => value.length > 3);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("every", () => {
        const dictionary = (0, dictionary_1.dict)({
            hello: "こんにちは",
            world: "世界",
        });
        (0, vitest_1.expect)(dictionary.every((key) => key.length > 3)).toBeTruthy();
        (0, vitest_1.expect)(dictionary.every((key, value) => value.length > 3)).toBeFalsy();
    });
    (0, vitest_1.test)("some", () => {
        const dictionary = (0, dictionary_1.dict)({
            hello: "こんにちは",
            world: "世界",
        });
        (0, vitest_1.expect)(dictionary.some((key) => key.length < 3)).toBeFalsy();
        (0, vitest_1.expect)(dictionary.some((key, value) => value.length < 3)).toBeTruthy();
    });
});
//# sourceMappingURL=dictionary.test.js.map