"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const hash_1 = require("./hash");
(0, vitest_1.describe)("hashable tests", () => {
    (0, vitest_1.test)("文字列が同じならばhash値が等しい", () => {
        const h1 = "hello";
        const h2 = "hello";
        (0, vitest_1.expect)((0, hash_1.hash)(h1)).toBe((0, hash_1.hash)(h2));
    });
    (0, vitest_1.test)("文字列が異なるならばhash値が異なる", () => {
        const h1 = "hello";
        const h2 = "world";
        (0, vitest_1.expect)((0, hash_1.hash)(h1) === (0, hash_1.hash)(h2)).toBeFalsy();
    });
    (0, vitest_1.test)("数値が同じならばhash値が等しい", () => {
        const h1 = 100;
        const h2 = 100;
        (0, vitest_1.expect)((0, hash_1.hash)(h1)).toBe((0, hash_1.hash)(h2));
    });
    (0, vitest_1.test)("数値が異なるならばhash値が異なる", () => {
        const h1 = 100;
        const h2 = 101;
        (0, vitest_1.expect)((0, hash_1.hash)(h1) === (0, hash_1.hash)(h2)).toBeFalsy();
    });
    (0, vitest_1.test)("真偽値が同じならばhash値が等しい", () => {
        const h1 = true;
        const h2 = true;
        (0, vitest_1.expect)((0, hash_1.hash)(h1)).toBe((0, hash_1.hash)(h2));
    });
    (0, vitest_1.test)("真偽値が異なるならばhash値が異なる", () => {
        const h1 = true;
        const h2 = false;
        (0, vitest_1.expect)((0, hash_1.hash)(h1) === (0, hash_1.hash)(h2)).toBeFalsy();
    });
    (0, vitest_1.test)("型が異なるならばhash値が異なる", () => {
        const h1 = true;
        const h2 = "true";
        (0, vitest_1.expect)((0, hash_1.hash)(h1) === (0, hash_1.hash)(h2)).toBeFalsy();
    });
    (0, vitest_1.test)("nullとundefinedはhash値が等しい", () => {
        const h1 = undefined;
        const h2 = null;
        (0, vitest_1.expect)((0, hash_1.hash)(h1)).toBe((0, hash_1.hash)(h2));
    });
});
//# sourceMappingURL=hash.test.js.map