"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const some_1 = require("./some");
const none_1 = require("./none");
const result_1 = require("../result");
const equality_1 = require("../equality");
const clone_1 = require("../clone");
const codable_1 = require("../codable");
const optional_1 = require("./optional");
(0, vitest_1.describe)("Some Tests", () => {
    (0, vitest_1.test)("getで値を取り出せる", () => {
        const option = (0, some_1.some)(10);
        (0, vitest_1.expect)(option.get).toBe(10);
    });
    (0, vitest_1.test)("valueで値を取り出せる", () => {
        const option = (0, some_1.some)(10);
        (0, vitest_1.expect)(option.value).toBe(10);
    });
    (0, vitest_1.test)("getOrElseで値を取り出せる", () => {
        const option = (0, some_1.some)("hello");
        (0, vitest_1.expect)(option.getOrElse("world")).toBe("hello");
    });
    (0, vitest_1.test)("orElseでインスタンスがそのまま出てくる", () => {
        const option = (0, some_1.some)("hello");
        (0, vitest_1.expect)(option.orElse((0, some_1.some)("world"))).toEqual(option);
    });
    (0, vitest_1.test)("isDefinedはtrue", () => {
        const option = (0, some_1.some)("hello");
        (0, vitest_1.expect)(option.isDefined).toBeTruthy();
    });
    (0, vitest_1.test)("isEmptyはfalse", () => {
        const option = (0, some_1.some)("hello");
        (0, vitest_1.expect)(option.isEmpty).toBeFalsy();
    });
    (0, vitest_1.test)("mapで値を変換できる", () => {
        const option = (0, some_1.some)(10);
        (0, vitest_1.expect)(option.map((value) => value * 2)).toEqual((0, some_1.some)(20));
    });
    (0, vitest_1.test)("flatMapで値を変換できる", () => {
        const option = (0, some_1.some)(10);
        (0, vitest_1.expect)(option.flatMap((value) => (0, some_1.some)(value * 2))).toEqual((0, some_1.some)(20));
    });
    (0, vitest_1.test)("flatMapでnoneが帰ってきたらnoneを返す", () => {
        const option = (0, some_1.some)(10);
        (0, vitest_1.expect)(option.flatMap((_) => (0, none_1.none)())).toEqual((0, none_1.none)());
    });
    (0, vitest_1.test)("同じ値同士ならequalsでtrueを返す", () => {
        const option1 = (0, some_1.some)(10);
        const option2 = (0, some_1.some)(10);
        (0, vitest_1.expect)(option1.equals(option2)).toBeTruthy();
    });
    (0, vitest_1.test)("optional型でなくてもvalueと同じならequalsでtrueを返す", () => {
        const option1 = (0, some_1.some)(10);
        const value = 10;
        (0, vitest_1.expect)(option1.equals(value)).toBeTruthy();
    });
    (0, vitest_1.test)("equatableならばそれで比較する", () => {
        class E extends equality_1.Equatable {
            constructor(str, num) {
                super();
                this.str = str;
                this.num = num;
            }
        }
        const option1 = (0, some_1.some)(new E("hello", 10));
        const option2 = (0, some_1.some)(new E("hello", 10));
        (0, vitest_1.expect)(option1.equals(option2)).toBeTruthy();
    });
    (0, vitest_1.test)("異なる値同士ならequalsでfalseを返す", () => {
        const option1 = (0, some_1.some)(10);
        const option2 = (0, some_1.some)(20);
        (0, vitest_1.expect)(option1.equals(option2)).toBeFalsy();
    });
    (0, vitest_1.test)("引数に関数が与えられた場合はsome同士ならば関数の戻り値がtrueならばtrueを返す", () => {
        const option1 = (0, some_1.some)({ num: 3 });
        const option2 = (0, some_1.some)({ num: 3 });
        (0, vitest_1.expect)(option1.equals(option2, (a, b) => a.num === b.num)).toBeTruthy();
    });
    (0, vitest_1.test)("引数に関数が与えられた場合はsome同士ならば関数の戻り値がfalseならばfalseを返す", () => {
        const option1 = (0, some_1.some)({ num: 3 });
        const option2 = (0, some_1.some)({ num: 3 });
        (0, vitest_1.expect)(option1.equals(option2, () => false)).toBeFalsy();
    });
    (0, vitest_1.test)("noneと比較するとfalseを返す", () => {
        const option1 = (0, some_1.some)(10);
        const option2 = (0, none_1.none)();
        (0, vitest_1.expect)(option1.equals(option2)).toBeFalsy();
    });
    (0, vitest_1.test)("foldで第一引数の戻り値を返す", () => {
        const option = (0, some_1.some)(10);
        (0, vitest_1.expect)(option.fold(() => 0, (value) => value * 2)).toBe(20);
    });
    (0, vitest_1.test)("unwrapで値を取り出せる", () => {
        const option = (0, some_1.some)(10);
        (0, vitest_1.expect)(option.unwrap(new Error("Error"))).toBe(10);
    });
    (0, vitest_1.test)("toResultでsuccessに変換する", () => {
        const option = (0, some_1.some)(10);
        (0, vitest_1.expect)(option.toResult(new Error("error"))).toEqual((0, result_1.success)(10));
    });
    (0, vitest_1.test)("cloneでvalueをcloneする", () => {
        class C extends clone_1.Clonable {
            constructor(num) {
                super();
                this.num = num;
            }
        }
        const option = (0, some_1.some)(new C(10));
        const clone = option.clone;
        clone.value.num = 20;
        (0, vitest_1.expect)(option.value.num).toBe(10);
    });
    class J extends codable_1.Codable {
        constructor(num) {
            super();
            this.num = num;
        }
    }
    (0, vitest_1.test)("optionalでdecodeする", () => {
        var _a;
        const option = (0, some_1.some)(new J(0));
        const actual = option.decode({ num: 100 });
        (0, vitest_1.expect)((_a = actual.value) === null || _a === void 0 ? void 0 : _a.num).toBe(100);
    });
    (0, vitest_1.test)("nullをdecodeするとoptionalを返す", () => {
        const option = (0, some_1.some)(new J(0));
        const actual = option.decode(null);
        (0, vitest_1.expect)(actual).toBe((0, none_1.none)());
    });
    (0, vitest_1.test)("encodeするとvalueをencodeする", () => {
        const option = (0, some_1.some)(new J(1000));
        const actual = option.encode;
        (0, vitest_1.expect)(actual).toEqual({ num: 1000 });
    });
    (0, vitest_1.test)("要素で比較できる", () => {
        (0, vitest_1.expect)((0, some_1.some)(3).compare((0, some_1.some)(5))).toBeTruthy();
        (0, vitest_1.expect)((0, some_1.some)(5).compare((0, some_1.some)(3))).toBeFalsy();
    });
    (0, vitest_1.test)("noneと比較するとtrueを返す", () => {
        (0, vitest_1.expect)((0, some_1.some)(3).compare((0, optional_1.optional)(null))).toBeTruthy();
    });
});
//# sourceMappingURL=some.test.js.map