import { IEquatableObject } from "../equality";
import { IClonable } from "../clone";
import { Optional } from "../optional";
import { ClosedRange, OpenRange, Range } from "../range";
import { IDecodable, IEncodable } from "../codable";
import { IHashableObject } from "../hash";
export declare class Sequence<T> implements IEquatableObject<Sequence<T>>, IHashableObject<Sequence<T>>, IClonable<Sequence<T>>, IterableIterator<T>, IEncodable, IDecodable<Sequence<T>> {
    private readonly values;
    constructor(values: readonly T[]);
    /** 要素が空の場合のみtrueを返す */
    get isEmpty(): boolean;
    /** 要素が空の場合のみfalseを返す */
    get nonEmpty(): boolean;
    /** 最初の要素を取得する */
    get first(): Optional<T>;
    /** 最後の要素を取得する */
    get last(): Optional<T>;
    /** 要素を取り出す */
    get(index: number): T;
    /** 要素を取り出す */
    at(index: number): Optional<T>;
    /** 最後尾に値を追加する */
    append(value: T): Sequence<T>;
    /** 間に値を挿入する */
    insert(element: T, at: number): Sequence<T>;
    remove(at: number): Sequence<T>;
    /** 文字列の配列を結合する */
    join(separator: string): string;
    /** 要素を変換する */
    map<U>(f: (value: T, index: number, sequence: Sequence<T>) => U): Sequence<U>;
    /** 要素をフラットに変換する */
    flatMap<U>(f: (value: T, index: number, sequence: Sequence<T>) => Sequence<U>): Sequence<U>;
    /** unwrapしてsomeのみでmapを行う */
    compactMap<Option, TO>(f: (value: Option, index: number, sequence: Sequence<Option>) => TO): Sequence<TO>;
    /** 要素をフィルタリングする */
    filter(f: (value: T, index: number, sequence: Sequence<T>) => boolean): Sequence<T>;
    /** 連結する */
    concat(elements: Sequence<T>): Sequence<T>;
    /** 全ての要素が条件を満たす際にtrueを返す */
    every(f: (value: T, index: number, array: Sequence<T>) => boolean): boolean;
    /** どれかの要素が条件を満たす際にtrueを返す */
    some(f: (value: T, index: number, array: Sequence<T>) => boolean): boolean;
    /** ソートする。元のSequenceに影響はない */
    sorted(sorter?: ((a: T, b: T) => boolean) | ((a: T, b: T) => number)): Sequence<T>;
    /** 最大値を取得する */
    max(sorter?: (a: T, b: T) => boolean): Optional<T>;
    /** 最小値を取得する */
    min(sorter?: (a: T, b: T) => boolean): Optional<T>;
    /** 検索する */
    find(f: (value: T, index: number, array: Sequence<T>) => boolean): Optional<T>;
    /** index付きのSequenceを返す */
    get enumerated(): Sequence<{
        index: number;
        value: T;
    }>;
    /** 要素を逆順に並び替えたものを返す */
    get reversed(): Sequence<T>;
    /** 要素が存在するか判別する */
    contains(element: T): boolean;
    firstIndex(of: T): Optional<number>;
    /** 要素をランダムに並び替えたものを返す */
    get shuffled(): Sequence<T>;
    /** 入れ替えられる */
    swap(a: number, b: number): Sequence<T>;
    /** 範囲の要素を置き換える */
    replaceSubrange(subrange: Range<number> | ClosedRange<number> | OpenRange<number>, withNewElements: readonly T[] | Sequence<T>): Sequence<T>;
    /** 要素を集約する */
    reduce<U>(init: U, f: (previousValue: U, currentValue: T, currentIndex: number, array: readonly T[]) => U): U;
    /** 指定した範囲を取得する */
    slice(range: ClosedRange<number> | Range<number> | OpenRange<number>): Sequence<T>;
    /** ランダムな要素を取得する。cryptoを使用 */
    randomElement(): Optional<T>;
    equals(other: Sequence<T>): boolean;
    get clone(): Sequence<T>;
    /** 要素数を返す */
    get count(): number;
    get distinct(): Sequence<T>;
    [Symbol.iterator](): IterableIterator<T>;
    next(): IteratorResult<T>;
    get encode(): unknown;
    decode(object: unknown): Sequence<T>;
    get hashValue(): string;
    get toArray(): readonly T[];
}
export declare function seq<T>(array: readonly T[] | Sequence<T>): Sequence<T>;
export declare function seq<T>(...elements: readonly T[]): Sequence<T>;
