"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const sequence_1 = require("./sequence");
const assertion_test_1 = require("../equality/assertion.test");
const optional_1 = require("../optional");
const range_1 = require("../range");
const codable_1 = require("../codable");
(0, vitest_1.describe)("Sequence Tests", () => {
    (0, vitest_1.test)("getで要素を取得できる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        (0, vitest_1.expect)(sequence.get(0)).toBe(1);
        (0, vitest_1.expect)(sequence.get(1)).toBe(2);
        (0, vitest_1.expect)(sequence.get(2)).toBe(3);
    });
    (0, vitest_1.test)("atで要素を取得できる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        (0, vitest_1.expect)(sequence.at(-4)).toEqual((0, optional_1.none)());
        (0, vitest_1.expect)(sequence.at(-3)).toEqual((0, optional_1.some)(1));
        (0, vitest_1.expect)(sequence.at(-2)).toEqual((0, optional_1.some)(2));
        (0, vitest_1.expect)(sequence.at(-1)).toEqual((0, optional_1.some)(3));
        (0, vitest_1.expect)(sequence.at(0)).toEqual((0, optional_1.some)(1));
        (0, vitest_1.expect)(sequence.at(1)).toEqual((0, optional_1.some)(2));
        (0, vitest_1.expect)(sequence.at(2)).toEqual((0, optional_1.some)(3));
        (0, vitest_1.expect)(sequence.at(3)).toEqual((0, optional_1.none)());
    });
    (0, vitest_1.test)("countで要素数を取得できる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        (0, vitest_1.expect)(sequence.count).toBe(3);
    });
    (0, vitest_1.test)("要素空がない場合、isEmptyがfalseを返す", () => {
        (0, vitest_1.expect)((0, sequence_1.seq)([1]).isEmpty).toBeFalsy();
    });
    (0, vitest_1.test)("要素が空の場合isEmptyがtrueを返す", () => {
        (0, vitest_1.expect)((0, sequence_1.seq)([]).isEmpty).toBeTruthy();
    });
    (0, vitest_1.test)("要素空がない場合、nonEmptyがtrueを返す", () => {
        (0, vitest_1.expect)((0, sequence_1.seq)([1]).nonEmpty).toBeTruthy();
    });
    (0, vitest_1.test)("要素が空の場合nonEmptyがfalseを返す", () => {
        (0, vitest_1.expect)((0, sequence_1.seq)([]).nonEmpty).toBeFalsy();
    });
    (0, vitest_1.test)("firstで最初の要素が取れる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        (0, vitest_1.expect)(sequence.first).toEqual((0, optional_1.some)(1));
    });
    (0, vitest_1.test)("要素が空のときfirstでnoneが返される", () => {
        const sequence = (0, sequence_1.seq)([]);
        (0, vitest_1.expect)(sequence.first).toBe((0, optional_1.none)());
    });
    (0, vitest_1.test)("lastで最後の要素が取れる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        (0, vitest_1.expect)(sequence.last).toEqual((0, optional_1.some)(3));
    });
    (0, vitest_1.test)("要素が空のときlastでnoneが返される", () => {
        const sequence = (0, sequence_1.seq)([]);
        (0, vitest_1.expect)(sequence.last).toBe((0, optional_1.none)());
    });
    (0, vitest_1.test)("Iterable", () => {
        const sequence = (0, sequence_1.seq)(["hello", "my", "world"]);
        const expected = "hellomyworld";
        let actual = "";
        for (const element of sequence) {
            actual += element;
        }
        (0, vitest_1.expect)(actual).toBe(expected);
    });
    (0, vitest_1.test)("joinで文字列を連結できる", () => {
        const sequence = (0, sequence_1.seq)(["hello", "my", "world"]);
        const expected = "hello my world";
        const actual = sequence.join(" ");
        (0, vitest_1.expect)(actual).toBe(expected);
    });
    (0, vitest_1.test)("appendで値を追加できる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        const expected = (0, sequence_1.seq)([1, 2, 3, 4]);
        const actual = sequence.append(4);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("insertで間に値を追加できる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        const expected = (0, sequence_1.seq)([1, 2, 2.5, 3]);
        const actual = sequence.insert(2.5, 2);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("insertで最初に値を追加できる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        const expected = (0, sequence_1.seq)([0.5, 1, 2, 3]);
        const actual = sequence.insert(0.5, 0);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("insertで最後に値を追加できる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        const expected = (0, sequence_1.seq)([1, 2, 3, 4]);
        const actual = sequence.insert(4, 3);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("removeで指定した位置の要素を削除できる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        const expected = (0, sequence_1.seq)([1, 3]);
        const actual = sequence.remove(1);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("mapで値を変換できる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        const expected = (0, sequence_1.seq)([2, 4, 6]);
        const actual = sequence.map((n) => n * 2);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("flatMapで値を変換して平らにできる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        const expected = (0, sequence_1.seq)([1, 1, 2, 2, 3, 3]);
        const actual = sequence.flatMap((n) => (0, sequence_1.seq)([n, n]));
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("compactMapでnone値を除外して変換できる", () => {
        const sequence = (0, sequence_1.seq)((0, optional_1.some)(1), (0, optional_1.some)(2), (0, optional_1.none)(), (0, optional_1.some)(4));
        const expected = (0, sequence_1.seq)(3, 6, 12);
        const actual = sequence.compactMap((n) => n * 3);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("filter", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        const expected = (0, sequence_1.seq)([1, 3]);
        const actual = sequence.filter((n) => n % 2 === 1);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("reduce", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        const expected = 6;
        const actual = sequence.reduce(0, (acc, n) => acc + n);
        (0, vitest_1.expect)(actual).toBe(expected);
    });
    (0, vitest_1.test)("全ての要素がtrueのときにeveryがtrueを返す", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        const actual = sequence.every((n) => n > 0);
        (0, vitest_1.expect)(actual).toBeTruthy();
    });
    (0, vitest_1.test)("どれか一つの要素がfalseのときにeveryがfalseを返す", () => {
        const sequence = (0, sequence_1.seq)([1, -1, 3]);
        const actual = sequence.every((n) => n > 0);
        (0, vitest_1.expect)(actual).toBeFalsy();
    });
    (0, vitest_1.test)("どれか1つの要素がtrueのときにsomeがtrueを返す", () => {
        const sequence = (0, sequence_1.seq)([-1, 2, 3]);
        const actual = sequence.some((n) => n < 0);
        (0, vitest_1.expect)(actual).toBeTruthy();
    });
    (0, vitest_1.test)("どれか一つの要素がfalseのときにsomeがfalseを返す", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        const actual = sequence.some((n) => n < 0);
        (0, vitest_1.expect)(actual).toBeFalsy();
    });
    (0, vitest_1.test)("numberでsortが可能", () => {
        const sequence = (0, sequence_1.seq)([2, 1, 3]);
        const expected = (0, sequence_1.seq)([1, 2, 3]);
        const actual = sequence.sorted();
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("引数でsortが可能", () => {
        const sequence = (0, sequence_1.seq)([2, 1, 3]);
        const expected = (0, sequence_1.seq)([3, 2, 1]);
        const actual = sequence.sorted((a, b) => a > b);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    class N extends codable_1.Codable {
        constructor(value) {
            super();
            this.value = value;
        }
        compare(compared) {
            return this.value < compared.value;
        }
    }
    (0, vitest_1.test)("Comparable Objectでsortが可能", () => {
        const sequence = (0, sequence_1.seq)([new N(2), new N(1), new N(3)]);
        const expected = (0, sequence_1.seq)([new N(1), new N(2), new N(3)]);
        const actual = sequence.sorted();
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("maxを取得できる", () => {
        const sequence = (0, sequence_1.seq)([new N(2), new N(1), new N(3)]);
        const expected = (0, optional_1.some)(new N(3));
        const actual = sequence.max();
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("minを取得できる", () => {
        const sequence = (0, sequence_1.seq)([new N(2), new N(1), new N(3)]);
        const expected = (0, optional_1.some)(new N(1));
        const actual = sequence.min();
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("含むかを確認できる", () => {
        const sequence = (0, sequence_1.seq)([new N(1), new N(2), new N(3)]);
        (0, vitest_1.expect)(sequence.contains(new N(2))).toBeTruthy();
        (0, vitest_1.expect)(sequence.contains(new N(4))).toBeFalsy();
    });
    (0, vitest_1.test)("最初の合致する要素のindexを探す", () => {
        const sequence = (0, sequence_1.seq)([new N(1), new N(2), new N(3)]);
        const expected = (0, optional_1.some)(1);
        const actual = sequence.firstIndex(new N(2));
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("findで検索できる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        const expected = (0, optional_1.some)(2);
        const actual = sequence.find((n) => n === 2);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("enumeratedでindex付きのが取得できる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        const expected = (0, sequence_1.seq)([
            { index: 0, value: 1 },
            { index: 1, value: 2 },
            { index: 2, value: 3 },
        ]);
        const actual = sequence.enumerated;
        (0, vitest_1.expect)(actual).toEqual(expected);
    });
    (0, vitest_1.test)("reversedで逆順に並び替えられる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3, 4, 5]);
        const expected = (0, sequence_1.seq)([5, 4, 3, 2, 1]);
        const actual = sequence.reversed;
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("open rangeでsliceできる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3, 4, 5]);
        const expected = (0, sequence_1.seq)([3, 4, 5]);
        const actual = sequence.slice((0, range_1.from)(2));
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("rangeでsliceできる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3, 4, 5]);
        const expected = (0, sequence_1.seq)([3, 4]);
        const actual = sequence.slice((0, range_1.from)(2).until(4));
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("closed rangeでsliceできる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3, 4, 5]);
        const expected = (0, sequence_1.seq)([3, 4, 5]);
        const actual = sequence.slice((0, range_1.from)(2).to(4));
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("open rangeで入れ替えられる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 7, 4, 5]);
        const expected = (0, sequence_1.seq)([1, 2, 100, 101, 5]);
        const openRange = (0, range_1.from)(2);
        const actual = sequence.replaceSubrange(openRange, [100, 101, 5]);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("rangeで入れ替えられる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 7, 4, 5]);
        const expected = (0, sequence_1.seq)([1, 2, 100, 101, 5]);
        const range = (0, range_1.from)(2).until(4);
        const actual = sequence.replaceSubrange(range, [100, 101]);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("closed rangeで入れ替えられる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 7, 4, 5]);
        const expected = (0, sequence_1.seq)([1, 2, 100, 101, 5]);
        const closedRange = (0, range_1.from)(2).to(3);
        const actual = sequence.replaceSubrange(closedRange, [100, 101]);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("concatで連結できる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        const expected = (0, sequence_1.seq)([1, 2, 3, 4, 5, 6]);
        const actual = sequence.concat((0, sequence_1.seq)([4, 5, 6]));
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("encodableに準拠", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        const expected = [1, 2, 3];
        const actual = sequence.encode;
        (0, vitest_1.expect)(actual).toEqual(expected);
    });
    (0, vitest_1.test)("decodableに準拠", () => {
        const json = [{ value: 3 }, { value: 4 }, { value: 6 }];
        const expected = (0, sequence_1.seq)([new N(3), new N(4), new N(6)]);
        const actual = (0, sequence_1.seq)(new N(0)).decode(json);
        (0, vitest_1.expect)(actual).toEqual(expected);
    });
    (0, vitest_1.test)("distinctで重複を排除できる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3, 2, 3, 4, 5]);
        const expected = (0, sequence_1.seq)([1, 2, 3, 4, 5]);
        const actual = sequence.distinct;
        (0, vitest_1.expect)(actual).toEqual(expected);
    });
    (0, vitest_1.test)("swapで入れ替えられる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3, 4, 5]);
        const expected = (0, sequence_1.seq)([1, 4, 3, 2, 5]);
        const actual = sequence.swap(1, 3);
        (0, vitest_1.expect)(actual).toEqual(expected);
    });
});
//# sourceMappingURL=sequence.test.js.map