import { IEquatableObject, IEquatable } from "../equality";
import { IDecodable, IEncodable } from "../codable";
import { IClonable } from "../clone";
import { Sequence } from "../sequence";
import { Optional } from "../optional";
import { IHashable, IHashableObject } from "../hash";
export declare class Set<Element extends IHashable> implements IEquatableObject<Set<Element>>, IClonable<Set<Element>>, IEncodable, IHashableObject<Set<Element>>, IDecodable<Set<Element>>, IterableIterator<Element> {
    private readonly seq;
    constructor(seq: Sequence<Element>);
    /** 引数と集合の要素の積集合を返す */
    intersection(other: readonly Element[] | Sequence<Element> | Set<Element>): Set<Element>;
    /** 引数と集合の要素の和集合を返す */
    union(other: readonly Element[] | Sequence<Element> | Set<Element>): Set<Element>;
    /** 引数がこの集合の部分集合であればtrueを返す */
    isSubset(other: readonly Element[] | Sequence<Element> | Set<Element>): boolean;
    except(other: readonly Element[] | Sequence<Element> | Set<Element>): Set<Element>;
    /** 集合が空であればtrueを返す */
    get isEmpty(): boolean;
    /** 集合が空でなければtrueを返す */
    get nonEmpty(): boolean;
    get clone(): Set<Element>;
    /** 集合の要素数を返す */
    get count(): number;
    get toSeq(): Sequence<Element>;
    /** 集合に加える */
    insert(element: Element): Set<Element>;
    map<U extends IEquatable & IHashable>(f: (v: Element, i: number, set: Set<Element>) => U): Set<U>;
    flatMap<U extends IEquatable & IHashable>(f: (v: Element, i: number, set: Set<Element>) => Set<U>): Set<U>;
    compactMap<OPTION extends IHashable & IEquatable, TO extends IEquatable & IHashable>(f: (v: OPTION, i: number, set: Set<OPTION>) => TO): Set<TO>;
    filter(f: (v: Element, i: number, set: Set<Element>) => boolean): Set<Element>;
    reduce<U>(init: U, f: (acc: U, v: Element, i: number, set: Set<Element>) => U): U;
    every(f: (v: Element, i: number, set: Set<Element>) => boolean): boolean;
    some(f: (v: Element, i: number, set: Set<Element>) => boolean): boolean;
    sorted(comparator?: (a: Element, b: Element) => boolean): Sequence<Element>;
    max(comparator?: (a: Element, b: Element) => boolean): Optional<Element>;
    min(comparator?: (a: Element, b: Element) => boolean): Optional<Element>;
    contains(element: Element): boolean;
    find(f: (v: Element, i: number, set: Set<Element>) => boolean): Optional<Element>;
    [Symbol.iterator](): IterableIterator<Element>;
    next(): IteratorResult<Element>;
    get encode(): unknown;
    decode(object: unknown): Set<Element>;
    equals(other: Set<Element>): boolean;
    get hashValue(): string;
}
export declare function set<Element extends IEquatable & IHashable>(sequence: readonly Element[] | Sequence<Element> | Set<Element>): Set<Element>;
export declare function set<Element extends IEquatable & IHashable>(...elements: readonly Element[]): Set<Element>;
export declare function set<Element extends IEquatable & IHashable>(): Set<Element>;
