/** AutoWebSocket Setting */
export interface Config {
    /** send ping interval [ms] */
    interval?: number;
    protocols?: string | string[];
    /** wait pong time [ms] */
    waitTime?: number;
    /** how many times should we re-try when connect failed */
    tryTimes?: number;
}
export declare const defaultConfig: Config;
export interface IAutoWebSocket {
    onclose: null | ((this: WebSocket, ev: CloseEvent) => any);
    onerror: null | ((this: WebSocket, ev: Event) => any);
    onmessage: null | ((this: WebSocket, ev: MessageEvent<any>) => any);
    onopen: null | ((this: WebSocket, ev: Event) => any);
    binaryType: BinaryType;
    readonly bufferedAmount: number;
    readonly extensions: string;
    readonly protocol: string;
    readonly readyState: number;
    readonly url: string;
    close(code?: number, reason?: string): void;
    send(data: string | ArrayBufferLike | Blob | ArrayBufferView): void;
    addEventListener(type: keyof WebSocketEventMap, listener: (this: WebSocket, ev: CloseEvent | Event | MessageEvent<any>) => any, _?: boolean | AddEventListenerOptions): void;
}
export declare class AutoWebSocket implements IAutoWebSocket {
    private readonly URI;
    private readonly config;
    private intervalID;
    private isClosed;
    private isPinged;
    private ws;
    binaryType: BinaryType;
    private _onclose;
    private _onerror;
    private _onmessage;
    private _onopen;
    constructor(uri: string, config: Config);
    get bufferedAmount(): number;
    get extensions(): string;
    get protocol(): string;
    get readyState(): number;
    get url(): string;
    close(code?: number, reason?: string): void;
    send(data: string | ArrayBufferLike | Blob | ArrayBufferView): void;
    get onclose(): ((this: WebSocket, ev: CloseEvent) => any) | null;
    set onclose(newValue: ((this: WebSocket, ev: CloseEvent) => any) | null);
    get onerror(): ((this: WebSocket, ev: Event) => any) | null;
    set onerror(newValue: ((this: WebSocket, ev: Event) => any) | null);
    get onmessage(): ((this: WebSocket, ev: MessageEvent<any>) => any) | null;
    set onmessage(newValue: ((this: WebSocket, ev: MessageEvent<any>) => any) | null);
    set onopen(newValue: (this: WebSocket, ev: Event) => any);
    readonly addEventListener: (type: keyof WebSocketEventMap, listener: (this: WebSocket, ev: CloseEvent | Event | MessageEvent<any>) => any, _?: boolean | AddEventListenerOptions) => void;
    private readonly connect;
    private readonly setProperty;
    private readonly trySocketConnect;
    private readonly continuation;
    private readonly stopContinuation;
}
